import Recipe from "./Recipe";
import Task from "./Task";
import Game from "./Game";
import * as Util from "./Util";

import { observable } from "mobx";

export default class Worker {
	nonSerializedProperties = [
		"task", "state", "inventoryChangeListener"
	];

	game: Game;
	recipe: Recipe;
	@observable task: Task;
	@observable state: State = State.STOPPED;
	inventoryChangeListener;

	public constructor(game: Game, recipe: Recipe) {
		this.game = game;
		this.recipe = recipe;
	}

	public start() {
		this.tryBeginWork();

		this.game.inventory.onChange.push(this.inventoryChangeListener = () => {
			this.onInventoryChange();
		});
	}

	private tryBeginWork(): boolean {
		// We check if we have the ressourcess
		var hasRessources = this.recipe.items.every((i) => {
			return this.game.inventory.has(i.item, i.quantity);
		});

		if (!hasRessources) {
			return;
		}

		// Can we begin the production ?
		if (this.state != State.STOPPED) {
			return;
		}

		// We begin the production
		this.task = this.game.taskManager.addTask(() => {
			this.onTaskFinish();
		}, this.recipe.time);
		
		this.state = State.WORKING;

		// We remove all the ressources
		this.recipe.items.forEach((i) => {
			this.game.inventory.removeItem(i.item, i.quantity);
		});
	}

	private pause() {
		if (this.state == State.PAUSED) {
			return;
		}
		this.interrupt();

		this.state = State.PAUSED;
	}

	private interrupt() {
		if (this.task == null) {
			return;
		}

		this.game.taskManager.removeTask(this.task);
	}

	private onInventoryChange() {
		if (this.state != State.STOPPED) {
			return;
		}

		this.tryBeginWork();
	}

	private onTaskFinish() {
		// We add the result item
		this.game.inventory.addItem(this.recipe.result, this.recipe.quantity);

		this.state = State.STOPPED;
		// We try to do one cycle again
		this.tryBeginWork();
	}

	public delete() {
		this.interrupt();
		this.game.inventory.onChange.splice(
			this.game.inventory.onChange.indexOf(this.inventoryChangeListener), 1
		);
	}
}

enum State {
	WORKING = 0,
	PAUSED = 1,
	STOPPED = 2
}
